@@ -195,6 +195,28 @@ static void testFileData(KArchive *archive)
     QCOMPARE(QString::fromLatin1(contents.constData()), QString::fromLatin1(arr.constData()));
     delete dev;
 
+    // test read/seek/peek work fine
+    f = dir->file(QStringLiteral("test1"));
+    dev = f->createDevice();
+    contents = dev->peek(4);
+    QCOMPARE(contents, QByteArray("Hall"));
+    contents = dev->peek(2);
+    QCOMPARE(contents, QByteArray("Ha"));
+    dev->seek(2);
+    contents = dev->peek(2);
+    QCOMPARE(contents, QByteArray("ll"));
+    dev->seek(0);
+    contents = dev->read(2);
+    QCOMPARE(contents, QByteArray("Ha"));
+    contents = dev->peek(2);
+    QCOMPARE(contents, QByteArray("ll"));
+    dev->seek(1);
+    contents = dev->read(1);
+    QCOMPARE(contents, QByteArray("a"));
+    dev->seek(4);
+    contents = dev->read(1);
+    QCOMPARE(contents, QByteArray("o"));
+
     const KArchiveEntry *e = dir->entry(QStringLiteral("mediumfile"));
     QVERIFY(e && e->isFile());
     f = (KArchiveFile *)e;
@@ -350,20 +372,23 @@ void KArchiveTest::testEmptyFilename()
     QTest::ignoreMessage(QtWarningMsg, "KArchive: No file name specified");
     KTar tar(QLatin1String(""));
     QVERIFY(!tar.open(QIODevice::ReadOnly));
+    QCOMPARE(tar.errorString(), tr("No filename or device was specified"));
 }
 
 void KArchiveTest::testNullDevice()
 {
-    QIODevice *nil = 0;
+    QIODevice *nil = nullptr;
     QTest::ignoreMessage(QtWarningMsg, "KArchive: Null device specified");
     KTar tar(nil);
     QVERIFY(!tar.open(QIODevice::ReadOnly));
+    QCOMPARE(tar.errorString(), tr("No filename or device was specified"));
 }
 
 void KArchiveTest::testNonExistentFile()
 {
     KTar tar(QStringLiteral("nonexistent.tar.gz"));
     QVERIFY(!tar.open(QIODevice::ReadOnly));
+    QCOMPARE(tar.errorString(), tr("File %1 does not exist").arg("nonexistent.tar.gz"));
 }
 
 void KArchiveTest::testCreateTar_data()
@@ -472,7 +497,7 @@ void KArchiveTest::testReadTar() // testCreateTarGz must have been run first.
         QVERIFY(tar.open(QIODevice::ReadOnly));
 
         const KArchiveDirectory *dir = tar.directory();
-        QVERIFY(dir != 0);
+        QVERIFY(dir != nullptr);
         const QStringList listing = recursiveListEntries(dir, QLatin1String(""), WithUserGroup | WithTime);
 
 
@@ -648,7 +673,7 @@ void KArchiveTest::testTarMaxLength()
     QVERIFY(tar.open(QIODevice::ReadOnly));
 
     const KArchiveDirectory *dir = tar.directory();
-    QVERIFY(dir != 0);
+    QVERIFY(dir != nullptr);
     const QStringList listing = recursiveListEntries(dir, QLatin1String(""), WithUserGroup);
 
     QCOMPARE(listing[0], QString("mode=100644 user= group= path=aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa0000000098 type=file size=3"));
@@ -669,7 +694,7 @@ void KArchiveTest::testTarGlobalHeader()
     QVERIFY2(tar.open(QIODevice::ReadOnly), "global_header_test.tar.gz");
 
     const KArchiveDirectory *dir = tar.directory();
-    QVERIFY(dir != 0);
+    QVERIFY(dir != nullptr);
 
     const QStringList listing = recursiveListEntries(dir, QLatin1String(""), WithUserGroup);
 
@@ -687,7 +712,7 @@ void KArchiveTest::testTarPrefix()
     QVERIFY2(tar.open(QIODevice::ReadOnly), "tar_prefix_test.tar.gz");
 
     const KArchiveDirectory *dir = tar.directory();
-    QVERIFY(dir != 0);
+    QVERIFY(dir != nullptr);
 
     const QStringList listing = recursiveListEntries(dir, QLatin1String(""), WithUserGroup);
 
@@ -713,7 +738,7 @@ void KArchiveTest::testTarDirectoryForgotten()
     QVERIFY2(tar.open(QIODevice::ReadOnly), "tar_directory_forgotten.tar.gz");
 
     const KArchiveDirectory *dir = tar.directory();
-    QVERIFY(dir != 0);
+    QVERIFY(dir != nullptr);
 
     const QStringList listing = recursiveListEntries(dir, QLatin1String(""), WithUserGroup);
 
@@ -731,7 +756,7 @@ void KArchiveTest::testTarRootDir() // bug 309463
     QVERIFY2(tar.open(QIODevice::ReadOnly), qPrintable(tar.fileName()));
 
     const KArchiveDirectory *dir = tar.directory();
-    QVERIFY(dir != 0);
+    QVERIFY(dir != nullptr);
 
     const QStringList listing = recursiveListEntries(dir, QLatin1String(""), WithUserGroup);
     //qDebug() << listing.join("\n");
@@ -749,7 +774,7 @@ void KArchiveTest::testTarDirectoryTwice() // bug 206994
     QVERIFY(tar.open(QIODevice::ReadOnly));
 
     const KArchiveDirectory *dir = tar.directory();
-    QVERIFY(dir != 0);
+    QVERIFY(dir != nullptr);
 
     const QStringList listing = recursiveListEntries(dir, QLatin1String(""), WithUserGroup);
     //qDebug() << listing.join("\n");
@@ -831,6 +856,10 @@ void KArchiveTest::testCreateZipError()
     KZip zip(QDir::currentPath());
 
     QVERIFY(!zip.open(QIODevice::WriteOnly));
+    QCOMPARE(
+        zip.errorString(),
+        tr("QSaveFile creation for %1 failed: Filename refers to a directory")
+            .arg(QDir::currentPath()));
 }
 
 void KArchiveTest::testReadZipError()
@@ -846,6 +875,9 @@ void KArchiveTest::testReadZipError()
         KZip zip(QStringLiteral("broken.zip"));
 
         QVERIFY(!zip.open(QIODevice::ReadOnly));
+        QCOMPARE(
+            zip.errorString(),
+            tr("Invalid ZIP file. Unexpected end of file. (Error code: %1)").arg(1));
 
         QVERIFY(brokenZip.open(QIODevice::WriteOnly | QIODevice::Append));
 
@@ -855,6 +887,9 @@ void KArchiveTest::testReadZipError()
         brokenZip.close();
 
         QVERIFY(!zip.open(QIODevice::ReadOnly));
+        QCOMPARE(
+            zip.errorString(),
+            tr("Invalid ZIP file. Unexpected end of file. (Error code: %1)").arg(4));
     }
 
     QVERIFY(brokenZip.remove());
@@ -868,7 +903,7 @@ void KArchiveTest::testReadZip()
     QVERIFY(zip.open(QIODevice::ReadOnly));
 
     const KArchiveDirectory *dir = zip.directory();
-    QVERIFY(dir != 0);
+    QVERIFY(dir != nullptr);
 
     // ZIP has no support for per-file user/group, so omit them from the listing
     const QStringList listing = recursiveListEntries(dir, QLatin1String(""), 0);
@@ -948,7 +983,7 @@ void KArchiveTest::testZipMaxLength()
     QVERIFY(zip.open(QIODevice::ReadOnly));
 
     const KArchiveDirectory *dir = zip.directory();
-    QVERIFY(dir != 0);
+    QVERIFY(dir != nullptr);
     const QStringList listing = recursiveListEntries(dir, QLatin1String(""), 0);
 
     QCOMPARE(listing[0], QString("mode=100644 path=aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa0000000098 type=file size=3"));
@@ -976,7 +1011,7 @@ void KArchiveTest::testZipWithNonLatinFileNames()
     QVERIFY(zip.open(QIODevice::ReadOnly));
 
     const KArchiveDirectory *dir = zip.directory();
-    QVERIFY(dir != 0);
+    QVERIFY(dir != nullptr);
     const QStringList listing = recursiveListEntries(dir, QLatin1String(""), 0);
 
     QCOMPARE(listing.count(), 1);
@@ -1005,7 +1040,7 @@ void KArchiveTest::testZipWithOverwrittenFileName()
     QVERIFY(zip.open(QIODevice::ReadOnly));
 
     const KArchiveDirectory *dir = zip.directory();
-    QVERIFY(dir != 0);
+    QVERIFY(dir != nullptr);
     const QStringList listing = recursiveListEntries(dir, QLatin1String(""), 0);
 
     QCOMPARE(listing.count(), 1);
@@ -1049,7 +1084,7 @@ void KArchiveTest::testZipAddLocalDirectory()
         QVERIFY(zip.open(QIODevice::ReadOnly));
 
         const KArchiveDirectory *dir = zip.directory();
-        QVERIFY(dir != 0);
+        QVERIFY(dir != nullptr);
 
         const KArchiveEntry *e = dir->entry(file1);
         QVERIFY(e && e->isFile());
@@ -1080,7 +1115,7 @@ void KArchiveTest::testZipReadRedundantDataDescriptor()
     QVERIFY(zip.open(QIODevice::ReadOnly));
 
     const KArchiveDirectory *dir = zip.directory();
-    QVERIFY(dir != 0);
+    QVERIFY(dir != nullptr);
 
     const QByteArray fileData("aaaaaaaaaaaaaaa");
 
@@ -1130,7 +1165,7 @@ void KArchiveTest::testRcc()
     KRcc rcc(rccFile);
     QVERIFY(rcc.open(QIODevice::ReadOnly));
     const KArchiveDirectory *rootDir = rcc.directory();
-    QVERIFY(rootDir != 0);
+    QVERIFY(rootDir != nullptr);
     const KArchiveEntry *rrEntry = rootDir->entry(QStringLiteral("runtime_resource"));
     QVERIFY(rrEntry && rrEntry->isDirectory());
     const KArchiveDirectory *rrDir = static_cast<const KArchiveDirectory *>(rrEntry);
@@ -1206,7 +1241,7 @@ void KArchiveTest::testRead7Zip() // testCreate7Zip must have been run first.
         QVERIFY(k7zip.open(QIODevice::ReadOnly));
 
         const KArchiveDirectory *dir = k7zip.directory();
-        QVERIFY(dir != 0);
+        QVERIFY(dir != nullptr);
         const QStringList listing = recursiveListEntries(dir, QLatin1String(""), 0);
 
 #ifndef Q_OS_WIN
@@ -1331,7 +1366,7 @@ void KArchiveTest::test7ZipMaxLength()
     QVERIFY(k7zip.open(QIODevice::ReadOnly));
 
     const KArchiveDirectory *dir = k7zip.directory();
-    QVERIFY(dir != 0);
+    QVERIFY(dir != nullptr);
     const QStringList listing = recursiveListEntries(dir, QLatin1String(""), 0);
 
     QCOMPARE(listing[0], QString("mode=100644 path=aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa0000000098 type=file size=3"));
